// Fullscrn.cpp: Implementation of the Full_screen class.

#include "stdafx.h"
#include "mainfrm.h"  // CMainFrame* is passed to functions in this
                      // class. Full_screen must be a friend of the
                      // CMainFrame class


Full_screen::Full_screen()
{
   m_in_full_screen_mode = m_have_forced_maximized = false;
   m_toolbar_was_visible = m_statusbar_was_visible = FALSE;
}


bool Full_screen::Is_in_full_screen_mode() const
{
   return m_in_full_screen_mode;
}


// Function to ensure that the frame window is maximized, and to
// hide the status bar and tool bar, if necessary
void Full_screen::Setup_for_full_screen(CMainFrame* pMainFrame)
{
   if (!m_in_full_screen_mode)  // only make changes if not already
      {                         // in full-screen mode
      m_toolbar_was_visible = pMainFrame->m_wndToolBar.
                                          ShowWindow(SW_HIDE);
      m_statusbar_was_visible = pMainFrame->m_wndStatusBar.
                                          ShowWindow(SW_HIDE);

      // get the active view; for an MDI application,
      // first GetActiveView() call returns NULL

      CView* pView = pMainFrame->GetActiveView();
      const bool is_mdi_app = (pView == (CView*)0) ? true : false;
      if (is_mdi_app)  // find the active view for MDI application
         {
         pView = (((CMDIFrameWnd*)pMainFrame)->MDIGetActive())
                                             ->GetActiveView();
         }

      CRect rect;                    // get current window position
      pMainFrame->GetWindowRect(&rect);

      if ((rect.Width() < ::GetSystemMetrics(SM_CXSCREEN)) ||
          (rect.Height() < ::GetSystemMetrics(SM_CYSCREEN)))
         {
         pMainFrame->ShowWindow(SW_SHOWMAXIMIZED);
         m_have_forced_maximized = true;

         if (is_mdi_app)
            {
            // Make sure all other windows are updated first

            MSG msg;
            while (::PeekMessage(&msg,NULL, NULL, NULL, PM_REMOVE))
               {
#ifdef WIN32               
               ::AfxGetThread()->PumpMessage();
#else
               ::TranslateMessage(&msg);
               ::DispatchMessage(&msg);
#endif               
               }

            pView->Invalidate();  // now call view class' OnDraw()
            }
         }
      else                     // frame already fills the screen
         {
         pView->Invalidate();  // now call to OnDraw()
         }

      m_in_full_screen_mode = true;
      }

   return;
}


// Function to restore all screen areas after full screen mode
void Full_screen::Restore_all_window_areas(CMainFrame* pMainFrame)
{
   if (m_in_full_screen_mode)   // only make changes if currently
      {                         // in full-screen mode
      m_in_full_screen_mode = false;

      if (m_toolbar_was_visible)
         pMainFrame->m_wndToolBar.ShowWindow(SW_RESTORE);

      if (m_statusbar_was_visible)
         pMainFrame->m_wndStatusBar.ShowWindow(SW_RESTORE);

      if (m_have_forced_maximized)
         {
         pMainFrame->ShowWindow(SW_RESTORE); // restore window size
         m_have_forced_maximized = false;
         }

      // use rectangle larger than entire screen size. Origin for
      // RedrawWindow() is the top, left corner of the screen

      CRect rect(-200, -200, 2000, 2000);
      pMainFrame->RedrawWindow(&rect, NULL, RDW_ERASE | RDW_FRAME |
                 RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN);
      }

   return;
}
