#define WIN32_NO_STATUS
#include <windows.h>
#define NOEXTAPI
#include <wdbgexts.h>
#include <stdio.h>

#undef MAKELANGID
#undef PRIMARYLANGID
#undef SUBLANGID
#undef MAKELCID
#undef LANGIDFROMLCID
#undef SORTIDFROMLCID
#undef UInt32x32To64
#undef WIN32_NO_STATUS

namespace NT {
    extern "C" {

#include <ntddk.h>

NTSYSAPI
NTSTATUS
NTAPI
ZwCreateToken(
    OUT PHANDLE TokenHandle,
    IN ACCESS_MASK DesiredAccess,
    IN POBJECT_ATTRIBUTES ObjectAttributes,
    IN TOKEN_TYPE Type,
    IN PLUID AuthenticationId,
    IN PLARGE_INTEGER ExpirationTime,
    IN PTOKEN_USER User,
    IN PTOKEN_GROUPS Groups,
    IN PTOKEN_PRIVILEGES Privileges,
    IN PTOKEN_OWNER Owner,
    IN PTOKEN_PRIMARY_GROUP PrimaryGroup,
    IN PTOKEN_DEFAULT_DACL DefaultDacl,
    IN PTOKEN_SOURCE Source
    );

    }
}
using NT::NTSTATUS;

typedef struct {
    ULONG Present   : 1;
    ULONG Writeable : 1;
    ULONG User      : 1;
    ULONG WriteThru : 1;
    ULONG NoCache   : 1;
    ULONG Accessed  : 1;
    ULONG Dirty     : 1;
    ULONG PageSize  : 1;
    ULONG Global    : 1;
    ULONG Available : 3;
    ULONG Pfn       : 20;
} PTE, *PPTE;

BOOL EnablePrivilege(PCSTR name)
{
    TOKEN_PRIVILEGES priv = {1, {0, 0, SE_PRIVILEGE_ENABLED}};
    LookupPrivilegeValue(0, name, &priv.Privileges[0].Luid);

    HANDLE hToken;
    OpenProcessToken(GetCurrentProcess(),
                     TOKEN_ADJUST_PRIVILEGES, &hToken);

    AdjustTokenPrivileges(hToken, FALSE, &priv, sizeof priv, 0, 0);
    BOOL rv = GetLastError() == ERROR_SUCCESS;

    CloseHandle(hToken);
    return rv;
}

PVOID Default(TOKEN_INFORMATION_CLASS tic)
{
    HANDLE hToken;
    OpenProcessToken(GetCurrentProcess(),
                     TOKEN_QUERY | TOKEN_QUERY_SOURCE, &hToken);

    ULONG n;
    GetTokenInformation(hToken, tic, 0, 0, &n);
    PBYTE p = new BYTE[n];
    GetTokenInformation(hToken, tic, p, n, &n);

    CloseHandle(hToken);
    return p;
}

BOOL ImpersonateSystem()
{
    EnablePrivilege(SE_CREATE_TOKEN_NAME);

    TOKEN_USER user = {{0, 0}};
    SID_IDENTIFIER_AUTHORITY nt = SECURITY_NT_AUTHORITY;
    AllocateAndInitializeSid(&nt, 1, SECURITY_LOCAL_SYSTEM_RID,
                             0, 0, 0, 0, 0, 0, 0, &user.User.Sid);

    TOKEN_SOURCE source = {{'*','*','A','N','O','N','*','*'}};
    AllocateLocallyUniqueId(&source.SourceIdentifier);

    PTOKEN_STATISTICS stats =
        PTOKEN_STATISTICS(Default(TokenStatistics));

    NT::SECURITY_QUALITY_OF_SERVICE sqos =
        {sizeof sqos, NT::SecurityImpersonation,
         SECURITY_STATIC_TRACKING, FALSE};
    NT::OBJECT_ATTRIBUTES oa = {sizeof oa, 0, 0, 0, 0, &sqos};

    HANDLE hToken;
    NT::ZwCreateToken(&hToken, TOKEN_ALL_ACCESS, &oa,
                   TokenImpersonation,
                   NT::PLUID(&stats->AuthenticationId),
                   NT::PLARGE_INTEGER(&stats->ExpirationTime),
                   &user,
                   PTOKEN_GROUPS(Default(TokenGroups)),
                   PTOKEN_PRIVILEGES(Default(TokenPrivileges)),
                   PTOKEN_OWNER(Default(TokenOwner)),
                   PTOKEN_PRIMARY_GROUP(Default(TokenPrimaryGroup)),
                   PTOKEN_DEFAULT_DACL(Default(TokenDefaultDacl)),
                   &source);

    BOOL rv = ImpersonateLoggedOnUser(hToken);

    // Tolerate memory leaks
    CloseHandle(hToken);
    return rv;
}

PVOID MapSystemPage(HANDLE hSection, ULONG prot, PPTE pde, ULONG va)
{
    ULONG offset;

    PTE pte1 = pde[va >> 22];
    if (!pte1.Present) return 0;

    if (pte1.PageSize == 0) {
        PPTE table = PPTE(MapViewOfFile(hSection, FILE_MAP_READ, 0,
                                        pte1.Pfn << 12, 0x1000));
        PTE pte2 = table[(va >> 12) & 0x3ff];
        UnmapViewOfFile(table);
        if (!pte2.Present) return 0;

        offset = pte2.Pfn << 12;
    }
    else
        offset = (pte1.Pfn << 12) + (va & 0x3ff000);

    return MapViewOfFile(hSection, prot, 0, offset, PAGE_SIZE);
}

ULONG WDBGAPI CheckControlC()
{
    return FALSE;
}

HANDLE hSection;
PPTE pde;

ULONG WDBGAPI ReadMemory(ULONG va, PVOID p, ULONG n, PULONG m)
{
    *m = 0;

    for (ULONG y, x = va; x < va + n; x += y) {
        PVOID q = MapSystemPage(hSection, FILE_MAP_READ, pde, x);
        if (q == 0) return FALSE;

        y = min(n - *m, PAGE_SIZE - BYTE_OFFSET(x));
        memcpy(PBYTE(p) + *m, PBYTE(q) + BYTE_OFFSET(x), y);

        UnmapViewOfFile(q);

        *m += y;
    }
    return TRUE;
}

int main(int argc, char *argv[])
{
    ImpersonateSystem();

    NT::UNICODE_STRING name;
    NT::RtlInitUnicodeString(&name, L"\\Device\\PhysicalMemory");
    NT::OBJECT_ATTRIBUTES oa = {sizeof oa, 0, &name,
                                OBJ_CASE_INSENSITIVE, 0, 0};
    NT::ZwOpenSection(&hSection, SECTION_MAP_READ, &oa);

    pde = PPTE(MapViewOfFile(hSection, FILE_MAP_READ, 0, 0x30000,
                             0x1000));

    WINDBG_EXTENSION_APIS apis = {sizeof apis};
    apis.lpOutputRoutine = PWINDBG_OUTPUT_ROUTINE(printf);
    apis.lpCheckControlCRoutine = CheckControlC;
    apis.lpReadProcessMemoryRoutine = ReadMemory;

    OSVERSIONINFO ver = {sizeof ver};
    GetVersionEx(&ver);

    char args[50];
    sprintf(args, "%.20s %.20s",
            argc > 1 ? argv[1] : "", argc > 2 ? argv[2] : "");

    HMODULE kdext = LoadLibrary("kdextx86.dll");

    PWINDBG_EXTENSION_DLL_INIT(GetProcAddress(kdext,
                                          "WinDbgExtensionDllInit"))
        (&apis, 0x0f, USHORT(ver.dwBuildNumber));

    PWINDBG_EXTENSION_ROUTINE(GetProcAddress(kdext, "ntfsdata"))
        (0, 0, 0, 0, args);

    return 0;
}
